<?php

namespace Drupal\commerce_order_item_sku\Plugin\QueueWorker;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes order item SKU updates.
 *
 * @QueueWorker(
 *   id = "commerce_order_item_sku",
 *   title = @Translation("Order Item SKU Worker"),
 *   cron = {"time" = 60}
 * )
 */
class OrderItemSkuWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->configFactory = $container->get('config.factory');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $order_item = $this->entityTypeManager
      ->getStorage('commerce_order_item')
      ->load($data['order_item_id']);

    $order_item_type_entity = $order_item->get('type')->entity;

    if (!$order_item_type_entity->hasTrait('commerce_order_item_sku')
      || isset($data['original_sku']) && $data['original_sku'] !== $order_item->get('sku')->value) {
      return;
    }

    $order_item->set('sku', $data['sku']);
    $order_item->save();
  }

}
