<?php

namespace Drupal\Tests\commerce_order_item_sku\Kernel;

use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_product\Entity\ProductVariation;

/**
 * Tests the commerce_order_item_sku_entity_update() function.
 *
 * @group commerce_order_item_sku
 */
class PurchasableEntityDeleteTest extends OrderItemSkuKernelTestBase {

  /**
   * The queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * The queue worker.
   *
   * @var \Drupal\Core\Queue\QueueWorkerInterface
   */
  protected $queueWorker;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->queue = $this->container->get('queue')->get('commerce_order_item_sku');
    $this->queueWorker = $this->container->get('plugin.manager.queue_worker')->createInstance('commerce_order_item_sku');

    $this->configFactory->getEditable('commerce_order_item_sku.settings')
      ->set('setting_event', 'purchased_entity_delete')
      ->set('sync_on_purchasable_entity_sku_update', FALSE)
      ->set('sync_on_purchased_entity_reference_update', FALSE)
      ->save();

    $variation = ProductVariation::create([
      'type' => 'default',
      'sku' => 'ORIGINAL_SKU',
    ]);
    $variation->save();

    $order_item = OrderItem::create([
      'type' => 'default',
      'purchased_entity' => $variation->id(),
    ]);
    $order_item->save();
  }

  /**
   * Tests the entity update.
   */
  public function testEntityUpdate() {
    $order_item = OrderItem::load(1);
    $this->assertNull(
      $order_item->get('sku')->value,
      'The order item SKU should be NULL before the variation is deleted.'
    );

    $variation = ProductVariation::load(1);
    $variation->delete();

    $this->queue->createQueue();
    $item = $this->queue->claimItem();
    $this->assertNotFalse(
      $item,
      'A queue item should have been created for synced SKU.'
    );
    $this->queueWorker->processItem($item->data);

    $order_item = OrderItem::load(1);
    $this->assertEquals(
      'ORIGINAL_SKU',
      $order_item->get('sku')->value,
      'The order item SKU should be updated after the variation SKU is updated.'
    );
  }

}
